#include "game.h"

#if DEBUG
#include <dlfcn.h>
#include <stdbool.h>
#include <stdio.h>
#include <sys/stat.h>
#include <time.h>

#define LIB "game.so"
#define LOCKFILE "game.so.lock"

static void *handle = NULL;
static long int last_mtime;
void (*update_character)(int window_height) = NULL;
void (*render_character)(SDL_Renderer *renderer) = NULL;

bool load_symbols(void) {
  struct stat st;
  if (stat(LOCKFILE, &st) == 0) {
    // If lock file exists, don't reload the library
    return true;
  }

  if (stat(LIB, &st) != 0) {
    perror("Error checking library stat: ");
    return false;
  }

  long int mtime = st.st_mtim.tv_sec * 1000000000 + st.st_mtim.tv_nsec;
  if (mtime == last_mtime) {
    return true;
  }

  last_mtime = mtime;

  if (handle) {
    close_library();
  }

  handle = dlopen(LIB, RTLD_NOW | RTLD_GLOBAL);
  if (!handle) {
    fprintf(stderr, "Failed to load library: %s\n", dlerror());
    return false;
  }

  dlerror(); // Clear errors

  *(void **)(&update_character) = dlsym(handle, "update_character");
  *(void **)(&render_character) = dlsym(handle, "render_character");

  char *error = dlerror();
  if (error != NULL) {
    fprintf(stderr, "Failed to load function symbol: %s", error);
    return false;
  }

  return true;
}

void close_library(void) {
  if (handle) {
    dlclose(handle);
    handle = NULL;
  }
}
#else
#include "SDL_rect.h"
#include "SDL_render.h"

typedef struct state State;
struct state {
  float delta;
  float speed;
  SDL_Rect character;
};

static State game = {
    .delta = 1.f / 60.f,
    .speed = 1600.f,
    .character = {.x = 375, .y = 0, .w = 50, .h = 50},
};

void update_character(int window_height) {
  game.character.y += game.speed * game.delta;
  if (game.character.y < 0 ||
      game.character.y + game.character.h > window_height) {
    game.speed *= -1.f;

    if (game.character.y < 0) {
      game.character.y = 0;
    } else {
      game.character.y = window_height - game.character.h;
    }
  }
}

void render_character(SDL_Renderer *renderer) {
  SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255);
  SDL_RenderFillRect(renderer, &game.character);
}
#endif
