#include "game.h"
#include <SDL2/SDL.h>
#include <SDL2/SDL_events.h>
#include <SDL2/SDL_render.h>
#include <SDL2/SDL_video.h>
#include <stdbool.h>

#ifdef NDEBUG
#define DEBUG 0
#else
#define DEBUG 1
#endif

#define WINDOW_WIDTH 800
#define WINDOW_HEIGHT 600

int main(void) {
  SDL_Init(SDL_INIT_EVERYTHING);

  SDL_Window *window =
      SDL_CreateWindow("Window", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                       WINDOW_WIDTH, WINDOW_HEIGHT, SDL_WINDOW_SHOWN);

  SDL_Renderer *renderer = SDL_CreateRenderer(
      window, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);

  bool running = true;

  SDL_Event event = {0};

#if DEBUG
  if (!load_symbols()) {
    fprintf(stderr, "Failed to load game library\n");
    return 1;
  }
#endif

  while (running) {
    while (SDL_PollEvent(&event)) {
      switch (event.type) {
      case SDL_QUIT:
        running = false;
        break;
      }
    }

#if DEBUG
    if (!load_symbols()) {
      fprintf(stderr, "Failed to load game library\n");
      return 1;
    }
#endif

    update_character(WINDOW_HEIGHT);

    SDL_SetRenderDrawColor(renderer, 255, 255, 255, 255);

    SDL_RenderClear(renderer);

    render_character(renderer);

    SDL_RenderPresent(renderer);
  }

#if DEBUG
  close_library();
#endif

  SDL_DestroyRenderer(renderer);

  SDL_DestroyWindow(window);

  SDL_Quit();

  return 0;
}
